@@ -187,18 +187,6 @@ class MalformedTemplateTestCase(TestCase):
                     "found a dot, on line 1"
                 ),
             ),
-            Case(
-                description="range start does not cast to int",
-                template="{% for x in (tag..4) %}{{ x }}{% endfor %}",
-                expect_exception=LiquidTypeError,
-                expect_msg=("expected an int or float, found 'goodbye', on line 1"),
-            ),
-            Case(
-                description="range end does not cast to int",
-                template="{% for x in (1..tag) %}{{ x }}{% endfor %}",
-                expect_exception=LiquidTypeError,
-                expect_msg=("expected an int or float, found 'goodbye', on line 1"),
-            ),
             Case(
                 description="missing equal in assignment tag",
                 template="{% assign x 5 %}",
@@ -265,6 +253,21 @@ class MalformedTemplateTestCase(TestCase):
                 expect_exception=LiquidSyntaxError,
                 expect_msg="unexpected '~', on line 1",
             ),
+            Case(
+                description="junk in `liquid` tag",
+                template="\n".join(
+                    [
+                        r"{{ 'hello' }}",
+                        r"{% liquid",
+                        r"echo 'foo'",
+                        r"aiu34bseu",
+                        r"%}",
+                    ]
+                ),
+                expect_exception=LiquidSyntaxError,
+                expect_render="hello\n",
+                expect_msg="unexpected tag 'aiu34bseu', on line 3",
+            ),
         ]
 
         self._test(test_cases, mode=Mode.STRICT)
